<?php

namespace edu\wisc\doit\uwphps;

/**
 * UseDetailsProvider is an interface defining how to load an {@link UserDetails} and provides constants for
 * supplied request header keys.
 */
abstract class UserDetailsProvider
{

    /** @var bool Flag indicating if headers are passed prefixed with 'HTTP_' */
    protected $httpHeaders;
    
    // General, shared constants relevant to UWMSN and UW-System login
    const EPPN = "eppn";
    const SHIB_SESSION_ID = 'Shib-Session-Id';
    const SHIB_SESSION_ID_HTTP = 'HTTP_SHIB_SESSION_ID';
    const UDDS = "udds";
    const SOURCE = "source";
    const ISIS_EMPLID = "isisEmplid";

    /** Delimiter used by multi-valued headers */
    const DELIMITER = ';';

    /**
     * UserDetailsProvider constructor.
     *
     * @param bool $http  true if Shibboleth attributes are delivered via HTTP request headers
     */
    public function __construct($http = false)
    {
        $this->httpHeaders = $http;
    }

    /**
     * Map Shibboleth header values to an associative array.
     *
     * @return UserDetails  currently authenticated user, null if not logged in
     */
    abstract public function loadUser();


    /**
     * Map a Shibboleth attribute to its associated HTTP header name.
     *
     * @param string $attribute attribute to map
     * @return string Shibboleth attribute name mapped to its equivalent HTTP header name
     *
     * @see https://wiki.shibboleth.net/confluence/display/SHIB2/NativeSPAttributeAccess NativeSPAttributeAccess
     */
    public static function httpHeaderFromAttribute($attribute)
    {
        return 'HTTP_' . strtoupper(str_replace('-', '_', $attribute));
    }
}
